<?php
/*
	Original Plugin Name: WP-Cumulus
	Original Plugin URI: http://www.roytanck.com/2008/03/15/wp-cumulus-released
	Original Description: Flash based Tag Cloud for WordPress
	Original Version: 1.21
	Author: Roy Tanck
	Author URI: http://www.roytanck.com
	
	Copyright 2008, Roy Tanck

	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.

	Prestashop Module name: PS-Cumulus
	ported by: paul r campbell
	port uri: http://www.ecartservice.net
	
	All credit to the original author :)

*/

if (!defined('_PS_VERSION_'))
	exit;
	
class PS_Cumulus extends Module
{
	private $_html = '';
	private $_postErrors = array();
	
	public function __construct()
	{
		$this->name = 'ps_cumulus';
		$this->tab = 'front_office_features';
		$this->version = '2.1';		
		
		parent::__construct();

		$this->displayName = $this->l('Cumulus for Prestashop');
		$this->description = $this->l('A nice flash tag cloud ported from the Wordress original by Roy Tanck (http://www.roytanck.com) - and absolutely FREE as all GPL code should be.');			
	}
	
	
	
	public function install()
	{  // Save out default values for the tag cloud
		$newoptions['nTags'] = '10';
		$newoptions['minFont'] = '10';
		$newoptions['maxFont'] = '18';
		$newoptions['width'] = '180';
		$newoptions['height'] = '180';
		$newoptions['tcolor'] = '555555';
		$newoptions['tcolor2'] = '777777';
		$newoptions['hicolor'] = '000000';
		$newoptions['bgcolor'] = 'ffffff';
		$newoptions['speed'] = '100';
		$newoptions['trans'] = 'false';
		$newoptions['distr'] = 'true';
		$newoptions['args'] = '';
		$newoptions['compmode'] = 'false';
		$newoptions['showwptags'] = 'true';
		$newoptions['mode'] = 'tags';
		$newoptions['sortby'] = 'times';
		$newoptions['sortdir'] = 'true';		
		
		if (!parent::install() OR
		!$this->registerHook('leftColumn') OR
		!$this->registerHook('header') OR
		!Configuration::updateValue($this->name.'_options', serialize($newoptions)))
		return false;
			return true;
	}
	
	public function uninstall()
	{
		if (!parent::uninstall() OR

		!Configuration::deleteByName($this->name.'_options'))
		
		return false;
		return true;	
	}
	
	public function getContent()
	{
		if (Tools::isSubmit('submit'))
		{			
			$this->_postValidation();
			
			if (!sizeof($this->_postErrors))
			{
				$newoptions = unserialize(Configuration::get($this->name.'_options'));
				$newoptions['nTags'] = Tools::getValue('nTags');
				$newoptions['sortby'] = Tools::getValue('sortby');
				$newoptions['sortdir'] = (Tools::getValue('sortdir') ? 'true' : 'false');				
				$newoptions['minFont'] = Tools::getValue('minFont');
				$newoptions['maxFont'] = Tools::getValue('maxFont');
				$newoptions['width'] = Tools::getValue('width');
				$newoptions['height'] = Tools::getValue('height');
				$newoptions['tcolor'] = Tools::getValue('tcolor');
				$newoptions['tcolor2'] = Tools::getValue('tcolor2');
				$newoptions['hicolor'] = Tools::getValue('hicolor');
				$newoptions['bgcolor'] = Tools::getValue('bgcolor');
				$newoptions['speed'] = Tools::getValue('speed');
				$newoptions['trans'] = (Tools::getValue('trans') ? 'true' : 'false');
				$newoptions['distr'] = (Tools::getValue('distr') ? 'true' : 'false');
				$newoptions['compmode'] = (Tools::getValue('compmode') ? 'true' : 'false');
				Configuration::updateValue($this->name.'_options', serialize($newoptions));
				$this->_html .= '<div class="conf confirm">'.$this->l('Settings updated').'</div>';
			}
			else
			{
				foreach ($this->_postErrors AS $err)
				{
					$this->_html .= '<div class="alert error">'.$err.'</div>';
				}
			}
		}
		
		$this->_displayForm();
		
		return $this->_html;
	}
	
	private function _displayForm()
	{
		$options = unserialize(Configuration::get($this->name.'_options'));
		$this->_html .= '
		<form action="'.$_SERVER['REQUEST_URI'].'" method="post">
			<fieldset>
				<legend><img src="../img/admin/cog.gif" alt="" class="middle" />'.$this->l('Standard Settings').'</legend>
					<label>'.$this->l('Number of Tags').'</label>
					<div class="margin-form">
						<input type="text" name="nTags" value="'.Tools::getValue('nTags', $options['nTags']).'" />
						<p class="clear">'.$this->l('Maximum number of tags to use (default 10)').'</p>
					</div>
					
					<label>'.$this->l('Sort by').'</label>
					<div class="margin-form">
						<select name = "sortby">
						<option value="none"'.($options['sortby'] == "none" ? ' SELECTED' : '').' >None</option>
						<option value="times"'.($options['sortby'] == "times" ? ' SELECTED' : '').' >Frequency</option>
						<option value="name"'.($options['sortby'] == "name" ? ' SELECTED' : '').' >Tag Name</option>
						</select>
						<p class="clear">'.$this->l('Sort the tags (when displayed as html)').'</p>
					</div>
					
					<label>'.$this->l('Ascending?').'</label>
					<div class="margin-form">
						<input type="checkbox" name="sortdir" value="'.Tools::getValue('sortdir', $options['sortdir']).'"' .
						($options['sortdir'] == "true" ? ' checked="checked"' : '') . ' />
						<p class="clear">'.$this->l('Choose sort direction as ascending (smallest first) or descending (no effect when sort by is \'none\')').'</p>
					</div>
					
					<label>'.$this->l('Minimum font size').'</label>
					<div class="margin-form">
						<input type="text" name="minFont" value="'.Tools::getValue('minFont', $options['minFont']).'" />
						<p class="clear">'.$this->l('Minimum font to use (will be scaled: default 10)').'</p>
					</div>
					<label>'.$this->l('Maximum font size').'</label>
					<div class="margin-form">
						<input type="text" name="maxFont" value="'.Tools::getValue('maxFont', $options['maxFont']).'" />
						<p class="clear">'.$this->l('Maximum font to use (will be scaled: default 18)').'</p>
					</div>
					<label>'.$this->l('Width of Cloud').'</label>
					<div class="margin-form">
						<input type="text" name="width" value="'.Tools::getValue('width', $options['width']).'" />
						<p class="clear">'.$this->l('Width in pixels (500 or more is recommended)').'</p>
					</div>
					<label>'.$this->l('Height of Cloud').'</label>
					<div class="margin-form">
						<input type="text" name="height" value="'.Tools::getValue('height', $options['height']).'" />
						<p class="clear">'.$this->l('Height in pixels (ideally around 3/4 of the width)').'</p>
					</div>
					<label>'.$this->l('Tag color').'</label>
					<div class="margin-form">
						<input type="text" name="tcolor" value="'.Tools::getValue('tcolor', $options['tcolor']).'" />
						<p class="clear">'.$this->l('hex color value without the # prefix (000000 for black, ffffff for white)').'</p>
					</div>
					<label>'.$this->l('Tag color for gradient').'</label>
					<div class="margin-form">
						<input type="text" name="tcolor2" value="'.Tools::getValue('tcolor2', $options['tcolor2']).'" />
						<p class="clear">'.$this->l('Distant color (000000 for black, ffffff for white)').'</p>
					</div>
					<label>'.$this->l('Highlight color').'</label>
					<div class="margin-form">
						<input type="text" name="hicolor" value="'.Tools::getValue('hicolor', $options['hicolor']).'" />
						<p class="clear">'.$this->l('Hover color (000000 for black, ffffff for white)').'</p>
					</div>
					<label>'.$this->l('Background color').'</label>
					<div class="margin-form">
						<input type="text" name="bgcolor" value="'.Tools::getValue('bgcolor', $options['bgcolor']).'" />
						<p class="clear">'.$this->l('Box color (000000 for black, ffffff for white)').'</p>
					</div>
					<label>'.$this->l('Use transparent mode').'</label>
					<div class="margin-form">
						<input type="checkbox" name="trans" value="'.(Tools::getValue('trans', $options['trans']) ? "true" : "false").'"' .
						($options['trans'] == "true" ? ' checked="checked"' : '') . ' />
						<p class="clear">'.$this->l('Switches on Flash\'s wmode-transparent setting').'</p>
					</div>
					<label>'.$this->l('Rotation speed').'</label>
					<div class="margin-form">
						<input type="text" name="speed" value="'.Tools::getValue('speed', $options['speed']).'" />
						<p class="clear">'.$this->l('Speed (percentage, default is 100)').'</p>
					</div>
					<label>'.$this->l('Even distribution').'</label>
					<div class="margin-form">
						<input type="checkbox" name="distr" value="'.(Tools::getValue('distr', $options['distr']) ? "true" : "false").'"' .
						($options['distr'] == "true" ? ' checked="checked"' : '') . ' />
						<p class="clear">'.$this->l('Places tags at equal intervals instead of random').'</p>
					</div>
					<label>'.$this->l('Compatibility mode').'</label>
					<div class="margin-form">
						<input type="checkbox" name="compmode" value="'.(Tools::getValue('compmode', $options['compmode']) ? "true" : "false").'"' .
						($options['compmode'] == "true" ? ' checked="checked"' : '') . ' />
						<p class="clear">'.$this->l('Enabling this option switches the plugin to a different way of embedding Flash into the page. Use this if your page has markup errors or if you\'re having trouble getting PS_Cumulus to display correctly.').'</p>
					</div>
				<input type="submit" name="submit" value="'.$this->l('Update').'" class="button" />			
			</fieldset>
		</form>';	
	}
	
	private function isHexColor($color)
	{
		return !$color OR preg_match('/[a-f 0-9]{6}/ui', $color);
	}
	
	private function _postValidation()
	{
		if (!Validate::isUnsignedInt(Tools::getValue('nTags')))
			$this->_postErrors[] = $this->l('Number of tags must be a postive integer');
		if (!Validate::isUnsignedInt(Tools::getValue('minFont')))
			$this->_postErrors[] = $this->l('Minimum font width must be a postive integer');
		if (!Validate::isUnsignedInt(Tools::getValue('maxFont')))
			$this->_postErrors[] = $this->l('Maximum font width must be a postive integer');	
		if (!Validate::isUnsignedInt(Tools::getValue('width')))
			$this->_postErrors[] = $this->l('Width must be a postive integer');
		if (!Validate::isUnsignedInt(Tools::getValue('height')))
			$this->_postErrors[] = $this->l('Height must be a postive integer');
		if (!$this->isHexColor(Tools::getValue('tcolor')))
			$this->_postErrors[] = $this->l('Tag color is invalid');
		if (!$this->isHexColor(Tools::getValue('tcolor2')))
			$this->_postErrors[] = $this->l('Optional Gradient color is invalid');
		if (!$this->isHexColor(Tools::getValue('hicolor')))
			$this->_postErrors[] = $this->l('Highlight color is invalid');
		if (!$this->isHexColor(Tools::getValue('bgcolor')))
			$this->_postErrors[] = $this->l('Background color is invalid');
		if (!Validate::isUnsignedInt(Tools::getValue('speed')))
			$this->_postErrors[] = $this->l('Speed must be a postive integer');
	}
	
	private function ps_cumulus_createflashcode($tagcloud)
	{
		// get the options
		$options = unserialize(Configuration::get($this->name.'_options'));
		$soname = "so";
		$divname = "wpcumuluscontent";
	
		$movie = $this->_path.'tagcloud.swf';
		$path = $this->_path;

		// add random seeds to so name and movie url to avoid collisions and force reloading (needed for IE)
		$soname .= rand(0,9999999);
		$movie .= '?r=' . rand(0,9999999);
		// write flash tag
		if( $options['compmode']!='true' ){
			$flashtag = '<!-- SWFObject embed by Geoff Stearns geoff@deconcept.com http://blog.deconcept.com/swfobject/ -->';	
			$flashtag .= '<script type="text/javascript" src="'.$path.'swfobject.js"></script>';
			$flashtag .= '<div id="'.$divname.'">';
			if( $options['showwptags'] == 'true' ){ $flashtag .= '<p>'; } else { $flashtag .= '<p style="display:none;">'; };
			$flashtag .= urldecode($tagcloud).'</div>';
			$flashtag .= '<script type="text/javascript">';
			$flashtag .= 'var '.$soname.' = new SWFObject("'.$movie.'", "tagcloudflash", "'.$options['width'].'", "'.$options['height'].'", "9", "#'.$options['bgcolor'].'");';
			if( $options['trans'] == 'true' ){
				$flashtag .= $soname.'.addParam("wmode", "transparent");';
			}
			$flashtag .= $soname.'.addParam("allowScriptAccess", "always");';
			$flashtag .= $soname.'.addVariable("tcolor", "0x'.$options['tcolor'].'");';
			$flashtag .= $soname.'.addVariable("tcolor2", "0x' . ($options['tcolor2'] == "" ? $options['tcolor'] : $options['tcolor2']) . '");';
			$flashtag .= $soname.'.addVariable("hicolor", "0x' . ($options['hicolor'] == "" ? $options['tcolor'] : $options['hicolor']) . '");';
			$flashtag .= $soname.'.addVariable("tspeed", "'.$options['speed'].'");';
			$flashtag .= $soname.'.addVariable("distr", "'.$options['distr'].'");';
			$flashtag .= $soname.'.addVariable("mode", "'.$options['mode'].'");';
			$flashtag .= $soname.'.addVariable("tagcloud", "'.urlencode('<tags>') . $tagcloud . urlencode('</tags>').'");';
			$flashtag .= $soname.'.write("'.$divname.'");';
			$flashtag .= '</script>';
		} else {
			$flashtag = '<object type="application/x-shockwave-flash" data="'.$movie.'" width="'.$options['width'].'" height="'.$options['height'].'">';
			$flashtag .= '<param name="movie" value="'.$movie.'" />';
			$flashtag .= '<param name="bgcolor" value="#'.$options['bgcolor'].'" />';
			$flashtag .= '<param name="AllowScriptAccess" value="always" />';
			if( $options['trans'] == 'true' ){
				$flashtag .= '<param name="wmode" value="transparent" />';
			}
			$flashtag .= '<param name="flashvars" value="';
			$flashtag .= 'tcolor=0x'.$options['tcolor'];
			$flashtag .= '&tcolor2=0x'.$options['tcolor2'];
			$flashtag .= '&hicolor=0x'.$options['hicolor'];
			$flashtag .= '&tspeed='.$options['speed'];
			$flashtag .= '&distr='.$options['distr'];
			$flashtag .= '&mode='.$options['mode'];
			$flashtag .= '&tagcloud='.urlencode('<tags>') . $tagcloud . urlencode('</tags>');			
			$flashtag .= '" />';
			$flashtag .= '<p>'.urldecode($tagcloud).'</p>';
			$flashtag .= '</object>';
			
		}
		return '<div class="pscumulus_block" style="background-color:#'.$options['bgcolor'].';">'.$flashtag.'</div>';	
	}
	
	private function getMainTags($id_lang, $nb = 10, $sortby = 'times', $sortdir = 'DESC')
	{
		global $cookie;
        Db::getInstance()->Execute('SET names UTF-8 ');
		return Db::getInstance()->Execute('
		SELECT t.name, COUNT(pt.id_tag) AS times
		FROM `'._DB_PREFIX_.'product_tag` pt
		LEFT JOIN `'._DB_PREFIX_.'tag` t ON t.id_tag = pt.id_tag
		LEFT JOIN `'._DB_PREFIX_.'product` p ON p.id_product = pt.id_product
		LEFT JOIN `'._DB_PREFIX_.'category_product` cp ON (cp.`id_product` = p.`id_product`)
		INNER JOIN `'._DB_PREFIX_.'category_group` ctg ON (ctg.`id_category` = cp.`id_category`)
		INNER JOIN `'._DB_PREFIX_.'customer_group` cg ON (cg.`id_group` = ctg.`id_group`)
		WHERE id_lang = '.intval($id_lang).'
		AND p.active = 1
		AND (cg.`id_customer` = '.intval($cookie->id_customer).' OR ctg.`id_group` = 1)
		GROUP BY t.id_tag ' .
		($sortby!='' ? 'ORDER BY '.$sortby.' '.$sortdir : ''). 
		' LIMIT 0, '.intval($nb+1));
	}
	
	public function hookHeader($params)
	{
		return '<link media="all" type="text/css" rel="stylesheet" href="'._MODULE_DIR_.$this->name.'/ps_cumulus.css"/>';
	}
	
	public function hookRightColumn($params)
	{
		$options = unserialize(Configuration::get($this->name.'_options'));
		$tagcloud ='';
		$numberTags = $options['nTags'];
		$tags = Tag::getMainTags((int)($params['cookie']->id_lang), $numberTags, ($options['sortby'] === 'none' ? '' :  $options['sortby']), ($options['sortdir'] == 'true' ? 'DESC' : 'ASC'));
		if (!sizeof($tags))
			return '';
		$maxFontSize = $options['maxFont'];
		$minFontSize = $options['minFont'];
		$maxNumber = 0;
		foreach ($tags AS $tag)
			$maxNumber = max($maxNumber,intval($tag['times']));
			
		$classPrefix = 'tag-link-';
		foreach ($tags AS $tag)
		{
			$tag['fontSize'] = floor(($maxFontSize * $tag['times']) / $maxNumber);
			if ($tag['fontSize'] < $minFontSize)
				$tag['fontSize'] = $minFontSize;
			// 2nd version: use CSS class
			$cssSizeclass = round($maxNumber/$tag['times']);
			if ($cssSizeclass>10)
				$tag['class'] = $classPrefix.'10';
			else
				$tag['class'] = $classPrefix.$cssSizeclass;
			$tagcloud .= urlencode('<a href ="'.__PS_BASE_URI__.'search.php?tag='.$tag['name'].'" class="'.$tag['class'].'" style="font-size:'.$tag['fontSize'].';">'.$tag['name'].'</a>').'%0A';
		}
		
		$code = $this->ps_cumulus_createflashcode($tagcloud);
		return $code;
	}
	
	public function hookLeftColumn($params)
	{
		return $this->hookRightColumn($params);
	}
	
	public function hookHome($params)
	{
		return $this->hookRightColumn($params);
	}
	
	public function hookTop($params)
	{
		return $this->hookRightColumn($params);
	}
	
	public function hookFooter($params)
	{
		return $this->hookRightColumn($params);
	}
	
}

